/*
 * Copyright (C) 2020 Purism SPC
 *
 * SPDX-License-Identifier: LGPL-2.1+
 */

#pragma once

#define FB_DBUS_NAME "org.sigxcpu.Feedback"
#define FB_DBUS_PATH "/org/sigxcpu/Feedback"

#define FB_DBUS_TYPE G_BUS_TYPE_SESSION
